/*
 * Written 1998 Jens Ch. Restemeier <jchrr@hrz.uni-bielefeld.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

/*
 * This is a viewer for Zak McKracken and Maniac Mansion Pictures.
 */

#include <stdio.h>
#include <stdlib.h>
#include <time.h>

#include <ggi/libggi.h>

int scr_width, scr_height;
ggi_graphtype graphtype;
ggi_visual_t vis_scr;
ggi_uint pal_idx[16];

unsigned char *s;

int open_video(int cols, int rows)
{
	int i;
	ggi_color pal[16];
	ggi_mode tm;
	ggiInit();
	
	/*
	 * Prepare a EGA 16-color palette
	 */
	pal[ 0].r=0x0000; pal[ 0].g=0x0000; pal[ 0].b=0x0000;
	pal[ 1].r=0x0000; pal[ 1].g=0x0000; pal[ 1].b=0x7FFF;
	pal[ 2].r=0x0000; pal[ 2].g=0x7FFF; pal[ 2].b=0x0000;
	pal[ 3].r=0x0000; pal[ 3].g=0x7FFF; pal[ 3].b=0x7FFF;
	pal[ 4].r=0x7FFF; pal[ 4].g=0x0000; pal[ 4].b=0x0000;
	pal[ 5].r=0x7FFF; pal[ 5].g=0x0000; pal[ 5].b=0x7FFF;
	pal[ 6].r=0x7FFF; pal[ 6].g=0x7FFF; pal[ 6].b=0x0000;
	pal[ 7].r=0x7FFF; pal[ 7].g=0x7FFF; pal[ 7].b=0x7FFF;
	pal[ 8].r=0x1FFF; pal[ 8].g=0x1FFF; pal[ 8].b=0x1FFF;
	pal[ 9].r=0x0000; pal[ 9].g=0x0000; pal[ 9].b=0xFFFF;
	pal[10].r=0x0000; pal[10].g=0xFFFF; pal[10].b=0x0000;
	pal[11].r=0x0000; pal[11].g=0xFFFF; pal[11].b=0xFFFF;
	pal[12].r=0xFFFF; pal[12].g=0x0000; pal[12].b=0x0000;
	pal[13].r=0xFFFF; pal[13].g=0x0000; pal[13].b=0xFFFF;
	pal[14].r=0xFFFF; pal[14].g=0xFFFF; pal[14].b=0x0000;
	pal[15].r=0xFFFF; pal[15].g=0xFFFF; pal[15].b=0xFFFF;

	vis_scr=ggiOpen(NULL);
	if (vis_scr==NULL)
		ggiPanic("unable to open screen-visual !\n");
	ggiSetGraphMode(vis_scr, cols, rows, cols, rows, GT_8BIT);
	ggiGetMode(vis_scr, &tm);
	scr_width=tm.visible.x;
	scr_height=tm.visible.y;

	for (i=0; i<16; i++)
		pal_idx[i]=ggiMapColor(vis_scr, pal[i]);

	return 1;
}

int close_video()
{
	ggiClose(vis_scr);
	ggiExit();
	return 1;
}

void main(int argc, char *argv[])
{
	FILE *f;
	unsigned int xx, yy, size;
	
	unsigned int font_pos, cols, rows;

	f=fopen(argv[1],"rb");
	fseek(f, 0, SEEK_END);
	size=ftell(f);
	fseek(f, 0, SEEK_SET);
	s=malloc(size);
	fread(s, size, 1, f);
	fclose(f);

	/* decode MM-resource */
#if 1
	for (font_pos=0; font_pos<size; font_pos++) {
		s[font_pos]^=0xFF;
	}
#endif
	cols	    =s[4] | (s[5]<<8);
	rows	    =s[6] | (s[7]<<8);
	font_pos    =s[10] | (s[11]<<8);

	if (cols>800) cols=800;

	open_video(cols, rows);

	xx=yy=0;
	while (xx<cols) {
		unsigned char cc;
		unsigned short ct;
		cc=s[font_pos++];
		ggiSetGCForeground(vis_scr, pal_idx[cc & 0x0F]);
		if ((cc & 0xF0)==0x00) {
			cc=s[font_pos++];
			ggiDrawBox(vis_scr, xx, yy, 16, cc);
			yy+=cc;
			xx+=yy/rows;
			yy%=rows;
		} else
		if (((cc & 0xF0)>0x00) && ((cc & 0xF0)<0x80)) {
			cc>>=4;
			ggiDrawBox(vis_scr, xx, yy, 16, cc);
			yy+=cc;
			xx+=yy/rows;
			yy%=rows;
		} else
		if ((cc & 0xF0)>=0x80) {
			cc&=0x7F;
			yy+=cc;
			xx+=yy/rows;
			yy%=rows;
		}
	}
	
	ggiGetc(vis_scr);
	close_video();
}
